module isajet_interface

  use kinds, only: default !NODEP!

  implicit none
  private

  external sugra, ssmssm, sstest

  public :: isasusy_set_parameters

  integer, parameter, public :: &
       & M_SUGRA = 1, M_GMSB = 2, NU_SUGRA = 3, GUT_SUGRA = 4, &
       & NM_GMSB = 5, SUGRA_NUR = 6, AMSB = 7

  integer, parameter, public :: MAX_ISAJET_CODE = 100

  type, public :: susy_input
     integer :: model_type
     real(kind=default) :: m0, m12, a0, tan_beta, sgn_mu, m_top
     real(kind=default) :: lambda, m_mes, n_5, c_gv
     real(kind=default) :: m32
     real(kind=default), dimension(4) :: xnrin
     real(kind=default), dimension(18) :: xnusug
     real(kind=default), dimension(8:14) :: xgmin
     real(kind=default), dimension(7) :: xsugin
  end type susy_input

  type, public :: susy_parameters
     type(susy_input) :: input
     real(kind=default) :: alpha_qed, sin2thw, alpha_s
     real(kind=default) :: mw, mz, ww, wz
     real(kind=default) :: m1, m2, m3
     real(kind=default) :: ae, au, ad
     real(kind=default) :: mu, b, q, m_h1_2, m_h2_2
     real(kind=default), dimension(32) :: mss
     real(kind=default) :: theta_top, theta_bot, theta_tau, alpha_h
     real(kind=default), dimension(4) :: m_neu
     real(kind=default), dimension(4,4) :: ev_neu
     real(kind=default), dimension(2) :: m_cha
     real(kind=default) :: gamma_l, gamma_r
     real(kind=default), dimension(MAX_ISAJET_CODE) :: width
  end type susy_parameters

contains

!-----------------------------------------------------------------------
! Set the SUSY parameters from input values, using RGE evolution

  subroutine isasusy_set_parameters (input, spar)

    type(susy_input), intent(in) :: input
    type(susy_parameters), intent(out) :: spar

! ISAJET common blocks
!
    common /SSLUN/ lout
    integer :: lout
    save /SSLUN/

!          Standard model parameters
!          AMUP,...,AMTP        = quark masses
!          AME,AMMU,AMTAU       = lepton masses
!          AMW,AMZ              = W,Z masses
!          GAMW,GAMZ            = W,Z widths
!          ALFAEM,SN2THW,ALFA3  = SM couplings
!          ALQCD4               = 4 flavor lambda
    common /SSSM/ amup,amdn,amst,amch,ambt,amtp,ame,ammu,amtau &
         & ,amw,amz,gamw,gamz,alfaem,sn2thw,alfa2,alfa3,alqcd4
    real :: amup,amdn,amst,amch,ambt,amtp,ame,ammu,amtau &
         & ,amw,amz,gamw,gamz,alfaem,sn2thw,alfa2,alfa3,alqcd4
    save /SSSM/

!          SUSY parameters
!          AMGLSS               = gluino mass
!          AMULSS               = up-left squark mass
!          AMELSS               = left-selectron mass
!          AMERSS               = right-slepton mass
!          AMNiSS               = sneutrino mass for generation i
!          TWOM1                = Higgsino mass = - mu
!          RV2V1                = ratio v2/v1 of vev's
!          AMTLSS,AMTRSS        = left,right stop masses
!          AMT1SS,AMT2SS        = light,heavy stop masses
!          AMBLSS,AMBRSS        = left,right sbottom masses
!          AMB1SS,AMB2SS        = light,heavy sbottom masses
!          AMLLSS,AMLRSS        = left,right stau masses
!          AML1SS,AML2SS        = light,heavy stau masses
!          AMZiSS               = signed mass of Zi
!          ZMIXSS               = Zi mixing matrix
!          AMWiSS               = signed Wi mass
!          GAMMAL,GAMMAR        = Wi left, right mixing angles
!          AMHL,AMHH,AMHA       = neutral Higgs h0, H0, A0 masses
!          AMHC                 = charged Higgs H+ mass
!          ALFAH                = Higgs mixing angle
!          AAT                  = stop trilinear term
!          THETAT               = stop mixing angle
!          AAB                  = sbottom trilinear term
!          THETAB               = sbottom mixing angle
!          AAL                  = stau trilinear term
!          THETAL               = stau mixing angle
!          AMGVSS               = gravitino mass
!          MTQ                  = top mass at weak scale
!          MBQ                  = bottom mass at weak scale
!          MLQ                  = tau mass at weak scale
    common /SSPAR/ amglss,amulss,amurss,amdlss,amdrss,amslss &
         & ,amsrss,amclss,amcrss,amblss,ambrss,amb1ss,amb2ss &
         & ,amtlss,amtrss,amt1ss,amt2ss,amelss,amerss,ammlss,ammrss &
         & ,amllss,amlrss,aml1ss,aml2ss,amn1ss,amn2ss,amn3ss &
         & ,twom1,rv2v1,amz1ss,amz2ss,amz3ss,amz4ss,zmixss(4,4) &
         & ,amw1ss,amw2ss &
         & ,gammal,gammar,amhl,amhh,amha,amhc,alfah,aat,thetat &
         & ,aab,thetab,aal,thetal,amgvss,mtq,mbq,mlq

    real :: amglss,amulss,amurss,amdlss,amdrss,amslss &
         & ,amsrss,amclss,amcrss,amblss,ambrss,amb1ss,amb2ss &
         & ,amtlss,amtrss,amt1ss,amt2ss,amelss,amerss,ammlss,ammrss &
         & ,amllss,amlrss,aml1ss,aml2ss,amn1ss,amn2ss,amn3ss &
         & ,twom1,rv2v1,amz1ss,amz2ss,amz3ss,amz4ss,zmixss &
         & ,amw1ss,amw2ss &
         & ,gammal,gammar,amhl,amhh,amha,amhc,alfah,aat,thetat &
         & ,aab,thetab,aal,thetal,amgvss,mtq,mbq,mlq
    real :: amziss(4)
    equivalence (amziss(1),amz1ss)
    save /SSPAR/

!          SM ident code definitions. These are standard ISAJET but
!          can be changed.
    integer, parameter :: &
         & IDUP= 1, IDDN= 2, IDST= 3, IDCH= 4, IDBT= 5, IDTP = 6, &
         & IDNE=11, IDE =12, IDNM=13, IDMU=14, IDNT=15, IDTAU=16, &
         & IDGL= 9, IDGM=10, IDW= 80, IDZ =90, IDH =81

!          SUSY ident code definitions. They are chosen to be similar
!          to those in versions < 6.50 but may be changed.
    integer, parameter :: &
         & ISUPL=21, ISDNL=22, ISSTL=23, ISCHL=24, ISBT1=25, ISTP1 =26, &
         & ISNEL=31, ISEL =32, ISNML=33, ISMUL=34, ISNTL=35, ISTAU1=36, &
         & ISUPR=41, ISDNR=42, ISSTR=43, ISCHR=44, ISBT2=45, ISTP2 =46, &
         & ISNER=51, ISER =52, ISNMR=53, ISMUR=54, ISNTR=55, ISTAU2=56, &
         & ISGL =29, &
         & ISZ1 =30, ISZ2 =40, ISZ3 =50, ISZ4 =60, ISW1 =39, ISW2  =49, &
         & ISHL =82, ISHH =83, ISHA =84, ISHC =86, &
         & ISGRAV=91, &
         & IDTAUL=10016, IDTAUR=20016

!          Frozen couplings from RG equations:
!     GSS( 1) = g_1        GSS( 2) = g_2        GSS( 3) = g_3
!     GSS( 4) = y_tau      GSS( 5) = y_b        GSS( 6) = y_t
!     GSS( 7) = M_1        GSS( 8) = M_2        GSS( 9) = M_3
!     GSS(10) = A_tau      GSS(11) = A_b        GSS(12) = A_t
!     GSS(13) = M_h1^2     GSS(14) = M_h2^2     GSS(15) = M_er^2
!     GSS(16) = M_el^2     GSS(17) = M_dnr^2    GSS(18) = M_upr^2
!     GSS(19) = M_upl^2    GSS(20) = M_taur^2   GSS(21) = M_taul^2
!     GSS(22) = M_btr^2    GSS(23) = M_tpr^2    GSS(24) = M_tpl^2
!     GSS(25) = mu         GSS(26) = B          GSS(27) = Y_N
!     GSS(28) = M_nr       GSS(29) = A_n
!          Masses:
!     MSS( 1) = glss     MSS( 2) = upl      MSS( 3) = upr
!     MSS( 4) = dnl      MSS( 5) = dnr      MSS( 6) = stl
!     MSS( 7) = str      MSS( 8) = chl      MSS( 9) = chr
!     MSS(10) = b1       MSS(11) = b2       MSS(12) = t1
!     MSS(13) = t2       MSS(14) = nuel     MSS(15) = numl
!     MSS(16) = nutl     MSS(17) = el-      MSS(18) = er-
!     MSS(19) = mul-     MSS(20) = mur-     MSS(21) = tau1
!     MSS(22) = tau2     MSS(23) = z1ss     MSS(24) = z2ss
!     MSS(25) = z3ss     MSS(26) = z4ss     MSS(27) = w1ss
!     MSS(28) = w2ss     MSS(29) = hl0      MSS(30) = hh0
!     MSS(31) = ha0      MSS(32) = h+
!          Unification:
!     MGUTSS  = M_GUT    GGUTSS  = g_GUT    AGUTSS  = alpha_GUT
    common /SUGMG/ mss(32),gss(29),mgutss,ggutss,agutss,ftgut, &
         &         fbgut,ftagut,fngut
    real ::  mss,gss,mgutss,ggutss,agutss,ftgut,fbgut,ftagut,fngut
    save /SUGMG/

!     XSUGIN contains the inputs to SUGRA:
!     XSUGIN(1) = M_0        XSUGIN(2) = M_(1/2)  XSUGIN(3) = A_0
!     XSUGIN(4) = tan(beta)  XSUGIN(5) = sgn(mu)  XSUGIN(6) = M_t
!     XSUGIN(7) = SUG BC scale
!     XGMIN(1) = LAM         XGMIN(2)  = M_MES    XGMIN(3)  = XN5
!     XGMIN(4) = tan(beta)   XGMIN(5)  = sgn(mu)  XGMIN(6) = M_t
!     XGMIN(7) = CGRAV       XGMIN(8)  =RSL       XGMIN(9)  = DEL_HD
!     XGMIN(10)  = DEL_HU    XGMIN(11) = DY       XGMIN(12) = N5_1
!     XGMIN(13)  = N5_2      XGMIN(14) = N5_3
!     XNRIN(1) = M_N3        XNRIN(2) = M_MAJ     XNRIN(3) = ANSS 
!     XNRIN(4) = M_N3SS
!     XISAIN contains the MSSMi inputs in natural order.
    common /SUGXIN/ xisain(24),xsugin(7),xgmin(14),xnrin(4)
    real :: xisain,xsugin,xgmin,xnrin
    save /SUGXIN/
    common /SUGPAS/ xtanb,msusy,amt,mgut,mu,g2,gp,v,vp,xw, &
         & a1mz,a2mz,asmz,ftamz,fbmz,b,sin2b,ftmt,g3mt,vev,higfrz, &
         & fnmz,amnrmj,nogood,ial3un,itachy,mhpneg,asm3
    real :: xtanb,msusy,amt,mgut,mu,g2,gp,v,vp,xw, &
         & a1mz,a2mz,asmz,ftamz,fbmz,b,sin2b,ftmt,g3mt,vev,higfrz, &
         & fnmz,amnrmj,asm3
    integer :: nogood,ial3un,itachy,mhpneg
    save /SUGPAS/

!     XNUSUG contains non-universal GUT scale soft terms for SUGRA:
!     XNUSUG(1)=M1 XNUSUG(2)=M2 XNUSUG(3)=M3
!     XNUSUG(4)=A_tau XNUSUG(5)=A_b XNUSUG(6)=A_t
!     XNUSUG(7)=m_Hd XNUSUG(8)=m_Hu XNUSUG(9)=m_eR XNUSUG(10)=m_eL
!     XNUSUG(11)=m_dR XNUSUG(12)=m_uR XNUSUG(13)=m_uL XNUSUG(14)=m_lR
!     XNUSUG(15)=m_lL XNUSUG(16)=m_bR XNUSUG(17)=m_tR XNUSUG(18)=m_tL

    common /SUGNU/ xnusug(18)
    real :: xnusug
    save /SUGNU/
!          ISAPW1 is used to check whether ALDATA is loaded
    common /ISAPW/ isapw1
    character(len=30) :: isapw1
    save /ISAPW/
!    character(len=80) :: fname

!   ISASUSY output
!          MXSS         =  maximum number of modes
!          NSSMOD       = number of modes
!          ISSMOD       = initial particle
!          JSSMOD       = final particles
!          GSSMOD       = width
!          BSSMOD       = branching ratio
!          MSSMOD       = decay matrix element pointer
!          LSSMOD       = logical flag used internally by SSME3

    integer, parameter :: MXSS = 1000
    common /SSMODE/ nssmod,issmod(MXSS),jssmod(5,MXSS),gssmod(MXSS) &
         & ,bssmod(MXSS),mssmod(MXSS),lssmod
    integer :: nssmod,issmod,jssmod,mssmod
    real :: gssmod,bssmod
    logical :: lssmod
    save /SSMODE/

    real :: m0,mhf,a0,tanb,sgnmu,mt,xlamgm,xmesgm,xn5gm,ampl,xcmgv
    integer :: imodel,inusug,imodin
    integer :: j,k,iallow,iitest
    real(kind=default) :: pi,gpx,sin2w,alemi,as

    character(len=40) :: versn,visaje
    character(len=30) :: isapw2
    save ISAPW2

    integer, parameter :: NOUT=33
    integer :: idout(NOUT)
    data idout/ &
         & idtp,isgl,isupl,isdnl,isstl,ischl,isbt1,istp1,isupr,isdnr, &
         & isstr,ischr,isbt2,istp2,isel,ismul,istau1,isnel,isnml,isntl, &
         & iser,ismur,istau2,isz1,isz2,isz3,isz4,isw1,isw2, &
         & ishl,ishh,isha,ishc/
    data ampl/2.4E18/
!          ISAPW2 is used to check whether ALDATA is loaded
    data isapw2/'ALDATA REQUIRED BY FORTRAN G,H'/

    if (isapw1 /= isapw2) then
       stop "isajet_interface: block data 'ALDATA' not loaded"
    end if

    select case (input%model_type)
    case (M_SUGRA, GUT_SUGRA, NU_SUGRA, SUGRA_NUR)
       imodel = 1
       m0    = input%m0
       mhf   = input%m12
       a0    = input%a0
       tanb  = input%tan_beta
       sgnmu = input%sgn_mu
       mt    = input%m_top
       select case (input%model_type)
       case (GUT_SUGRA)
          ial3un = 1                       
       case (NU_SUGRA, SUGRA_NUR)
          xnusug = input%xnusug                ! non-universal parameters
          xnrin  = input%xnrin                 ! right-handed neutrinos
          xsugin = input%xsugin                ! non-universal high scale
       end select
    case (M_GMSB, NM_GMSB)
       imodel = 2
       m0    = input%lambda
       mhf   = input%m_mes
       a0    = input%n_5
       tanb  = input%tan_beta
       sgnmu = input%sgn_mu
       mt    = input%m_top
       xgmin(7) = input%c_gv
       amgvss = m0 * mhf * xgmin(7) / sqrt(3.) / ampl   ! gravitino mass
       select case (input%model_type)
       case (M_GMSB)
          xgmin(8)    = 1.
       case (NM_GMSB)
          xgmin(8:14) = input%xgmin(8:14)
       end select
    case (AMSB)
       imodel = 7
       m0    = input%m0
       mhf   = input%m32
       a0    = 0.
       tanb  = input%tan_beta
       sgnmu = input%sgn_mu
       mt    = input%m_top
    case default
       print *, 'model_type', input%model_type
       stop "isajet_interface: invalid model type"
    end select

    ! Solve RGE: GUT -> EWSB
    call sugra (m0,mhf,a0,tanb,sgnmu,mt,imodel)
    
    ! EW checks
    if (mhpneg == 1)  nogood = 3
    select case (nogood)
    case (1)
       stop "isajet_interface: tachyonic particles"
    case (2)
       stop "isajet_interface: no electroweak symmetry breaking"
    case (3)
       stop "isajet_interface: m(H_P)^2 < 0"
    case (4)
       stop "isajet_interface: Yukawa coupling > 10"
    case (5)
       if (imodel==1) stop "isajet_interface: neutralino-1 is not LSP (SUGRA)"
    case (6)
    case (7)
       stop "isajet_interface: XT EWSB bad"
    case (8)
       stop "isajet_interface: m(H_L)^2 < 0"
    case (-1)
       stop "isajet_interface: no RGE solution found"
    end select
    if (itachy /= 0) &
         & print *, "isajet_interface: tachyonic sleptons at GUT scale"

    ! Transfer output
    spar%input = input

    pi    = 4.*atan(1._default)
    gpx   = sqrt(.6_default)*gss(1)
    sin2w = gpx**2/(gss(2)**2+gpx**2)
    alemi = 4*pi/gss(2)**2/sin2w
    as    = gss(3)**2/4/pi

    spar%alpha_qed = 1 / alemi ! SM parameters [recalculated]
    spar%sin2thw   = sin2w
    spar%alpha_s   = as

    spar%m1        = gss(7)    ! SUSY parameters at low scale
    spar%m2        = gss(8)
    spar%m3        = gss(9)
    spar%mu        = mu
    spar%b         = b
    spar%q         = higfrz
    spar%m_h1_2    = gss(13)
    spar%m_h2_2    = gss(14)

    spar%mss       = mss        ! sfermion masses

    spar%theta_top = thetat     ! sfermion mixing
    spar%theta_bot = thetab
    spar%theta_tau = thetal

    spar%alpha_h   = alfah      ! Higgs mixing

    spar%m_neu = (/ amz1ss,amz2ss,amz3ss,amz4ss /)  ! neutralinos
    spar%ev_neu = zmixss

    spar%m_cha = (/ amw1ss,amw2ss /) ! charginos
    spar%gamma_l = gammal
    spar%gamma_r = gammar

    ! SUSY decays
    call ssmssm (xisain(1),xisain(2),xisain(3), &
         & xisain(4),xisain(5),xisain(6),xisain(7),xisain(8),xisain(9), &
         & xisain(10),xisain(11),xisain(12),xisain(13),xisain(14), &
         & xisain(15),xisain(16),xisain(17),xisain(18),xisain(19), &
         & xisain(20),xisain(21),xisain(22),xisain(23),xisain(24), &
         & mt,iallow,imodel)

    if(iallow /= 0)  print *, "isajet_interface: neutralino-1 is not LSP"

    call sstest (iallow)
    iitest=iallow/2
    if (mod(iitest,2) /=0) print *, "isajet_interface: Z->nn exceeds bounds"
    iitest=iitest/2
    if (mod(iitest,2) /=0) print *, "isajet_interface: Z->charginos allowed"
    iitest=iitest/2
    if (mod(iitest,2) /=0) print *, "isajet_interface: Z->n1n2 too big"
    iitest=iitest/2
    if (mod(iitest,2) /=0) print *, "isajet_interface: Z->squarks/sleptons allowed"
    iitest=iitest/2
    if (mod(iitest,2) /=0) print *, "isajet_interface: Z -> Z* h exceeds bound"
    iitest=iitest/2
    if(mod(iitest,2) /=0) print *, "isajet_interface: Z -> hA allowed"

    do k = 1, MAX_ISAJET_CODE
       spar%width (k) = sum (gssmod(1:nssmod), issmod(1:nssmod)==k)
    end do

!     ! Chargino 1
!     print *, 'chargino widths'
!     do k = 1, nssmod
!        if (issmod(k)==39) then
!           print *, jssmod(1,k), jssmod(2,k), jssmod(3,k), gssmod(k), bssmod(k)
!        end if
!     end do
!     stop

    spar%mw        = amw       ! SM parameters [ISAJET hard-wired]
    spar%mz        = amz
    spar%ww        = gamw
    spar%wz        = gamz

  end subroutine isasusy_set_parameters
  

!-----------------------------------------------------------------------
! Return character name for ID, assuming the default IDENT codes
! are used in /SSTYPE/.

  function ssid (id)
    character(len=5) :: ssid
    integer, intent(in) :: id

    common /SSLUN/ lout
    integer lout
    save /SSLUN/
    character(len=5) :: label(-120:120)
    save label
    integer j

    data label(0)/'     '/      

    data (label(j),j=1,10) &
         & /'UP   ','DN   ','ST   ','CH   ','BT   ','TP   ' &
         & ,'ERROR','ERROR','GL   ','GM   '/
    data (label(j),j=-1,-10,-1) &
         &/'UB   ','DB   ','SB   ','CB   ','BB   ','TB   ' &
         &,'ERROR','ERROR','ERROR','ERROR'/

    data (label(j),j=11,20) &
         &/'NUE  ','E-   ','NUM  ','MU-  ','NUT  ','TAU- ' &
         &,'ERROR','ERROR','ERROR','ERROR'/ 
    data (label(j),j=-11,-20,-1) &
         &/'ANUE ','E+   ','ANUM ','MU+  ','ANUT ','TAU+ ' &
         &,'ERROR','ERROR','ERROR','ERROR'/

    data (label(j),j=21,30) &
         &/'UPL  ','DNL  ','STL  ','CHL  ','BT1  ','TP1  ' &
         &,'ERROR','ERROR','GLSS ','Z1SS '/
    data (label(j),j=-21,-30,-1) &
         &/'UBL  ','DBL  ','SBL  ','CBL  ','BB1  ','TB1  ' &
         &,'ERROR','ERROR','ERROR','ERROR'/

    data (label(j),j=31,40) &
         &/'NUEL ','EL-  ','NUML ','MUL- ','NUTL ','TAU1-' &
         &,'ERROR','ERROR','W1SS+','Z2SS '/
    data (label(j),j=-31,-40,-1) &
         &/'ANUEL','EL+  ','ANUML','MUL+ ','ANUTL','TAU1+' &
         &,'ERROR','ERROR','W1SS-','ERROR'/

    data (label(j),j=41,50) &
         &/'UPR  ','DNR  ','STR  ','CHR  ','BT2  ','TP2  ' &
         &,'ERROR','ERROR','W2SS+','Z3SS '/
    data (label(j),j=-41,-50,-1) &
         &/'UBR  ','DBR  ','SBR  ','CBR  ','BB2  ','TB2  ' &
         &,'ERROR','ERROR','W2SS-','ERROR'/

    data (label(j),j=51,60) &
         &/'NUER ','ER-  ','NUMR ','MUR- ','NUTR ','TAU2-' &
         &,'ERROR','ERROR','ERROR','Z4SS '/
    data (label(j),j=-51,-60,-1) &
         &/'ANUEL','ER+  ','ANUMR','MUR+ ','ANUTR','TAU2+' &
         &,'ERROR','ERROR','ERROR','ERROR'/

    data (label(j),j=82,86) &
         &/'HL0  ','HH0  ','HA0  ','ERROR','H+   '/
    data label(-86)/'H-   '/

    data label(80)/'W+   '/,label(-80)/'W-   '/,label(90)/'Z0   '/
    data label(91)/'GVSS '/
    data label(120)/'PI+  '/,label(-120)/'PI-  '/

    select case (iabs(id))
    case (-120:120)
       ssid = label(id)
    case default
       print *, 'SSID: ID = ',ID
       stop "isajet_interface: Unknown ID code"
    end select
  end function ssid

end module isajet_interface
